@@ -38,9 +38,10 @@ __all__ = ['fft', 'ifft', 'rfft', 'irfft', 'hfft', 'ihfft', 'rfftn',
 from numpy.core import (array, asarray, zeros, swapaxes, shape, conjugate,
                         take, sqrt)
 from . import fftpack_lite as fftpack
+from .helper import _FFTCache
 
-_fft_cache = {}
-_real_fft_cache = {}
+_fft_cache = _FFTCache(max_size_in_mb=100, max_item_count=32)
+_real_fft_cache = _FFTCache(max_size_in_mb=100, max_item_count=32)
 
 
 def _raw_fft(a, n=None, axis=-1, init_function=fftpack.cffti,
@@ -54,12 +55,13 @@ def _raw_fft(a, n=None, axis=-1, init_function=fftpack.cffti,
         raise ValueError("Invalid number of FFT data points (%d) specified."
                          % n)
 
-    try:
-        # Thread-safety note: We rely on list.pop() here to atomically
-        # retrieve-and-remove a wsave from the cache.  This ensures that no
-        # other thread can get the same wsave while we're using it.
-        wsave = fft_cache.setdefault(n, []).pop()
-    except (IndexError):
+    # We have to ensure that only a single thread can access a wsave array
+    # at any given time. Thus we remove it from the cache and insert it
+    # again after it has been used. Multiple threads might create multiple
+    # copies of the wsave array. This is intentional and a limitation of
+    # the current C code.
+    wsave = fft_cache.pop_twiddle_factors(n)
+    if wsave is None:
         wsave = init_function(n)
 
     if a.shape[axis] != n:
@@ -85,7 +87,7 @@ def _raw_fft(a, n=None, axis=-1, init_function=fftpack.cffti,
     # As soon as we put wsave back into the cache, another thread could pick it
     # up and start using it, so we must not do this until after we're
     # completely done using it ourselves.
-    fft_cache[n].append(wsave)
+    fft_cache.put_twiddle_factors(n, wsave)
 
     return r
 
@@ -119,6 +121,7 @@ def fft(a, n=None, axis=-1, norm=None):
         used.
     norm : {None, "ortho"}, optional
         .. versionadded:: 1.10.0
+
         Normalization mode (see `numpy.fft`). Default is None.
 
     Returns
@@ -169,6 +172,10 @@ def fft(a, n=None, axis=-1, norm=None):
              1.14383329e-17 +1.22460635e-16j,
              -1.64863782e-15 +1.77635684e-15j])
 
+    In this example, real input has an FFT which is Hermitian, i.e., symmetric
+    in the real part and anti-symmetric in the imaginary part, as described in
+    the `numpy.fft` documentation:
+
     >>> import matplotlib.pyplot as plt
     >>> t = np.arange(256)
     >>> sp = np.fft.fft(np.sin(t))
@@ -177,10 +184,6 @@ def fft(a, n=None, axis=-1, norm=None):
     [<matplotlib.lines.Line2D object at 0x...>, <matplotlib.lines.Line2D object at 0x...>]
     >>> plt.show()
 
-    In this example, real input has an FFT which is Hermitian, i.e., symmetric
-    in the real part and anti-symmetric in the imaginary part, as described in
-    the `numpy.fft` documentation.
-
     """
 
     a = asarray(a).astype(complex, copy=False)
@@ -229,6 +232,7 @@ def ifft(a, n=None, axis=-1, norm=None):
         axis is used.
     norm : {None, "ortho"}, optional
         .. versionadded:: 1.10.0
+
         Normalization mode (see `numpy.fft`). Default is None.
 
     Returns
@@ -306,6 +310,7 @@ def rfft(a, n=None, axis=-1, norm=None):
         used.
     norm : {None, "ortho"}, optional
         .. versionadded:: 1.10.0
+
         Normalization mode (see `numpy.fft`). Default is None.
 
     Returns
@@ -400,6 +405,7 @@ def irfft(a, n=None, axis=-1, norm=None):
         axis is used.
     norm : {None, "ortho"}, optional
         .. versionadded:: 1.10.0
+
         Normalization mode (see `numpy.fft`). Default is None.
 
     Returns
@@ -461,33 +467,36 @@ def irfft(a, n=None, axis=-1, norm=None):
 
 def hfft(a, n=None, axis=-1, norm=None):
     """
-    Compute the FFT of a signal which has Hermitian symmetry (real spectrum).
+    Compute the FFT of a signal that has Hermitian symmetry, i.e., a real
+    spectrum.
 
     Parameters
     ----------
     a : array_like
         The input array.
     n : int, optional
-        Length of the transformed axis of the output.
-        For `n` output points, ``n//2+1`` input points are necessary.  If the
-        input is longer than this, it is cropped.  If it is shorter than this,
-        it is padded with zeros.  If `n` is not given, it is determined from
-        the length of the input along the axis specified by `axis`.
+        Length of the transformed axis of the output. For `n` output
+        points, ``n//2 + 1`` input points are necessary.  If the input is
+        longer than this, it is cropped.  If it is shorter than this, it is
+        padded with zeros.  If `n` is not given, it is determined from the
+        length of the input along the axis specified by `axis`.
     axis : int, optional
         Axis over which to compute the FFT. If not given, the last
         axis is used.
     norm : {None, "ortho"}, optional
-        .. versionadded:: 1.10.0
         Normalization mode (see `numpy.fft`). Default is None.
 
+        .. versionadded:: 1.10.0
+
     Returns
     -------
     out : ndarray
         The truncated or zero-padded input, transformed along the axis
         indicated by `axis`, or the last one if `axis` is not specified.
         The length of the transformed axis is `n`, or, if `n` is not given,
-        ``2*(m-1)`` where ``m`` is the length of the transformed axis of the
-        input. To get an odd number of output points, `n` must be specified.
+        ``2*m - 2`` where ``m`` is the length of the transformed axis of
+        the input. To get an odd number of output points, `n` must be
+        specified, for instance as ``2*m - 1`` in the typical case,
 
     Raises
     ------
@@ -502,10 +511,12 @@ def hfft(a, n=None, axis=-1, norm=None):
     Notes
     -----
     `hfft`/`ihfft` are a pair analogous to `rfft`/`irfft`, but for the
-    opposite case: here the signal has Hermitian symmetry in the time domain
-    and is real in the frequency domain. So here it's `hfft` for which
-    you must supply the length of the result if it is to be odd:
-    ``ihfft(hfft(a), len(a)) == a``, within numerical accuracy.
+    opposite case: here the signal has Hermitian symmetry in the time
+    domain and is real in the frequency domain. So here it's `hfft` for
+    which you must supply the length of the result if it is to be odd.
+
+    * even: ``ihfft(hfft(a, 2*len(a) - 2) == a``, within roundoff error,
+    * odd: ``ihfft(hfft(a, 2*len(a) - 1) == a``, within roundoff error.
 
     Examples
     --------
@@ -538,32 +549,32 @@ def hfft(a, n=None, axis=-1, norm=None):
 
 def ihfft(a, n=None, axis=-1, norm=None):
     """
-    Compute the inverse FFT of a signal which has Hermitian symmetry.
+    Compute the inverse FFT of a signal that has Hermitian symmetry.
 
     Parameters
     ----------
     a : array_like
         Input array.
     n : int, optional
-        Length of the inverse FFT.
-        Number of points along transformation axis in the input to use.
-        If `n` is smaller than the length of the input, the input is cropped.
-        If it is larger, the input is padded with zeros. If `n` is not given,
-        the length of the input along the axis specified by `axis` is used.
+        Length of the inverse FFT, the number of points along
+        transformation axis in the input to use.  If `n` is smaller than
+        the length of the input, the input is cropped.  If it is larger,
+        the input is padded with zeros. If `n` is not given, the length of
+        the input along the axis specified by `axis` is used.
     axis : int, optional
         Axis over which to compute the inverse FFT. If not given, the last
         axis is used.
     norm : {None, "ortho"}, optional
-        .. versionadded:: 1.10.0
         Normalization mode (see `numpy.fft`). Default is None.
 
+        .. versionadded:: 1.10.0
+
     Returns
     -------
     out : complex ndarray
         The truncated or zero-padded input, transformed along the axis
         indicated by `axis`, or the last one if `axis` is not specified.
-        If `n` is even, the length of the transformed axis is ``(n/2)+1``.
-        If `n` is odd, the length is ``(n+1)/2``.
+        The length of the transformed axis is ``n//2 + 1``.
 
     See also
     --------
@@ -572,10 +583,12 @@ def ihfft(a, n=None, axis=-1, norm=None):
     Notes
     -----
     `hfft`/`ihfft` are a pair analogous to `rfft`/`irfft`, but for the
-    opposite case: here the signal has Hermitian symmetry in the time domain
-    and is real in the frequency domain. So here it's `hfft` for which
-    you must supply the length of the result if it is to be odd:
-    ``ihfft(hfft(a), len(a)) == a``, within numerical accuracy.
+    opposite case: here the signal has Hermitian symmetry in the time
+    domain and is real in the frequency domain. So here it's `hfft` for
+    which you must supply the length of the result if it is to be odd:
+
+    * even: ``ihfft(hfft(a, 2*len(a) - 2) == a``, within roundoff error,
+    * odd: ``ihfft(hfft(a, 2*len(a) - 1) == a``, within roundoff error.
 
     Examples
     --------
@@ -638,8 +651,8 @@ def fftn(a, s=None, axes=None, norm=None):
         Input array, can be complex.
     s : sequence of ints, optional
         Shape (length of each transformed axis) of the output
-        (`s[0]` refers to axis 0, `s[1]` to axis 1, etc.).
-        This corresponds to `n` for `fft(x, n)`.
+        (``s[0]`` refers to axis 0, ``s[1]`` to axis 1, etc.).
+        This corresponds to ``n`` for ``fft(x, n)``.
         Along any axis, if the given shape is smaller than that of the input,
         the input is cropped.  If it is larger, the input is padded with zeros.
         if `s` is not given, the shape of the input along the axes specified
@@ -651,6 +664,7 @@ def fftn(a, s=None, axes=None, norm=None):
         performed multiple times.
     norm : {None, "ortho"}, optional
         .. versionadded:: 1.10.0
+
         Normalization mode (see `numpy.fft`). Default is None.
 
     Returns
@@ -756,6 +770,7 @@ def ifftn(a, s=None, axes=None, norm=None):
         axis is performed multiple times.
     norm : {None, "ortho"}, optional
         .. versionadded:: 1.10.0
+
         Normalization mode (see `numpy.fft`). Default is None.
 
     Returns
@@ -831,8 +846,8 @@ def fft2(a, s=None, axes=(-2, -1), norm=None):
         Input array, can be complex
     s : sequence of ints, optional
         Shape (length of each transformed axis) of the output
-        (`s[0]` refers to axis 0, `s[1]` to axis 1, etc.).
-        This corresponds to `n` for `fft(x, n)`.
+        (``s[0]`` refers to axis 0, ``s[1]`` to axis 1, etc.).
+        This corresponds to ``n`` for ``fft(x, n)``.
         Along each axis, if the given shape is smaller than that of the input,
         the input is cropped.  If it is larger, the input is padded with zeros.
         if `s` is not given, the shape of the input along the axes specified
@@ -844,6 +859,7 @@ def fft2(a, s=None, axes=(-2, -1), norm=None):
         that a one-dimensional FFT is performed.
     norm : {None, "ortho"}, optional
         .. versionadded:: 1.10.0
+
         Normalization mode (see `numpy.fft`). Default is None.
 
     Returns
@@ -940,6 +956,7 @@ def ifft2(a, s=None, axes=(-2, -1), norm=None):
         that a one-dimensional FFT is performed.
     norm : {None, "ortho"}, optional
         .. versionadded:: 1.10.0
+
         Normalization mode (see `numpy.fft`). Default is None.
 
     Returns
@@ -1019,6 +1036,7 @@ def rfftn(a, s=None, axes=None, norm=None):
         axes are used, or all axes if `s` is also not specified.
     norm : {None, "ortho"}, optional
         .. versionadded:: 1.10.0
+
         Normalization mode (see `numpy.fft`). Default is None.
 
     Returns
@@ -1096,6 +1114,7 @@ def rfft2(a, s=None, axes=(-2, -1), norm=None):
         Axes over which to compute the FFT.
     norm : {None, "ortho"}, optional
         .. versionadded:: 1.10.0
+
         Normalization mode (see `numpy.fft`). Default is None.
 
     Returns
@@ -1153,6 +1172,7 @@ def irfftn(a, s=None, axes=None, norm=None):
         axis is performed multiple times.
     norm : {None, "ortho"}, optional
         .. versionadded:: 1.10.0
+
         Normalization mode (see `numpy.fft`). Default is None.
 
     Returns
@@ -1226,6 +1246,7 @@ def irfft2(a, s=None, axes=(-2, -1), norm=None):
         Default is the last two axes.
     norm : {None, "ortho"}, optional
         .. versionadded:: 1.10.0
+
         Normalization mode (see `numpy.fft`). Default is None.
 
     Returns
